/*
    FEARLUS model0-6-4: BitString.h
    Copyright (C) 1999-2002  Macaulay Institute

    This file is part of FEARLUS model0-6-4, an agent-based model of land use
    change.

    FEARLUS model0-6-4 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS model0-6-4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*


Interface for the bitString class. This is a class which stores a string of
bits, and provides mechanisms for performing bitwise operations on it. 

*/

#import <stdio.h>
#import <string.h>
#import <defobj.h>
#import <collections.h>
#import <objectbase/SwarmObject.h>

@class BitItem, NumberArray;

@interface BitString: SwarmObject {
  unsigned bitCount;
  unsigned bitsPerItem;
  id arr;
}

+create: (id)aZone setBitCount: (unsigned)aBitCount;
+create: (id)aZone setToString: (char *)aString;
+create: (id)aZone setToCopyOf: (BitString *)aBitString;
-(void)setBitCount: (unsigned)aBitCount;
-(unsigned)getBitCount;
-(void)setBit: (unsigned)n to: (int)value;
-(int)getBit: (unsigned)n;
-(void)toggleBit: (unsigned)n;
-(void)toggleEachBitUsingProbs: (NumberArray *)probs;
-(void)setEachBitTo: (int)value withProbability: (double)prob;
-(void)setToSubBitsOf: (BitString *)bitString
		 from: (unsigned)bit1 
		   to: (unsigned)bit2;
-(void)setToCopyOf: (BitString *)bitString;
-(void)setToConcatenationOf: (BitString *)bitString1
			and: (BitString *)bitString2;
-(void)setToParityOf: (BitString *)bitString1 and: (BitString *)bitString2;
-(void)setToOROf: (BitString *)bitString1 and: (BitString *)bitString2;
-(void)setToXOROf: (BitString *)bs1 and: (BitString *)bs2;
-(void)setToCrossoverOf: (BitString *)bitString1
		    and: (BitString *)bitString2
		     at: (unsigned)bit;
-(void)mutateWithProbability: (double)prob;
-(unsigned)getMatchWith: (BitString *)anItem;
-(unsigned)countOnes;
-(void)setToString: (char *)aString;
-(void)printToFile: (FILE *)fp;
-(void)printToString: (char *)aBuffer;
-print;
-(void)drop;
-(BitItem *)atOffset: (unsigned)i;
-(unsigned)getCount;

@end
